// Shift all the layers in an image by exactly half the image width.
// Both the visible AND invisible layers are shifted.
// All channels, Alpha channels, Masks, Adjustment Layers, ArtLayers are 
// shifted. Layersets/Layer Groups are also shifted.
//
// v1.7  by Eric Gerds  July 2007           http://www.pinlady.net/vr/
// Compatible with Photoshop CS/CS2/CS3
// Concept by Roger Howard
//
//
// Photoshop CS/CS2 bug discovered:
// Sometimes docref.width.value is not an integer (it is something
// like integer + 1e-13). This does not always happen, but it is possible.
// Hence, we must take Math.round(docref.width.value) to make sure
// we get an integer.


// Save photoshop state
function savestate(){
if (app.documents.length==0){return(false)};

// Save displayDialogs value
G.startdisplayDialogs = app.displayDialogs;

// Turn off all the dialogs
app.displayDialogs = DialogModes.NO;

G.strtRulerUnits = app.preferences.rulerUnits;
G.strtTypeUnits = app.preferences.typeUnits;
G.bgColor = app.backgroundColor;
G.fgColor = app.foregroundColor;

//  Set units make sure they are PIXELS
app.preferences.rulerUnits = Units.PIXELS;
app.preferences.typeUnits = TypeUnits.PIXELS;

return true;
}

// Restore photoshop state
function restorestate(){
if (app.documents.length==0){return(false)};

// Restore Photoshop Preferences
app.preferences.rulerUnits = G.strtRulerUnits;
app.preferences.typeUnits = G.strtTypeUnits;
app.backgroundColor = G.bgColor;
app.foregroundColor = G.fgColor;

// Restore dialogs
app.displayDialogs = G.startdisplayDialogs;

}

// Apply offset filter to the currently active layer only.
// For some strange reason, the
// activeLayer.applyOffset(hshift, 0, OffsetUndefinedAreas.WRAPAROUND)
// command is unable to shift any Alpha Channels, but the code output
// from the ScriptingListener can. So, for now we will use
// the ScriptingListener code.
function offs(h,v){
try{
  var desc3 = new ActionDescriptor();
  desc3.putInteger(charIDToTypeID("Hrzn"),h);
  desc3.putInteger(charIDToTypeID("Vrtc"),v);
  desc3.putEnumerated(charIDToTypeID("Fl  "),charIDToTypeID("FlMd"),
  charIDToTypeID("Wrp "));
  executeAction(charIDToTypeID("Ofst"), desc3, DialogModes.NO );
//  alert(app.activeDocument.activeLayer.name);
   }
catch(e){};
}


// Shift all layers (including adjustment layers) by half their width.
// This is a recursive function, so it can handle nested layersets.
//
// Note that activeLayer,activeChannels are properties of app.activeDocument
// and NOT of a layerSet. Hence, it is not valid to refer to
// layerSet[i].activeLayer or layerSet[i].activeChannels
//
// containeref = reference to a container object in a document. This may be
// the entire document itself (app.activeDocument), or it may be a layerSet.
// hshift = amount of the horizontal shift.
function shiftbyhalf(containeref,hshift){
var docref = app.activeDocument;

// For some reason, Photoshop decides to build histograms when
// certain commands are executed in a script, so we create a tiny
// selection area so the histogram is only built in this small area 
// - which is fast to do. This speeds up script execution.
var selregion = [[0,0],[1,0],[1,1],[0,1],[0,0]];
docref.selection.select(selregion,SelectionType.REPLACE);


var i;

// When containeref refers to the entire document (and NOT a layerSet),
// then remember active layer, active channels, and visibility of each channel.
if (containeref == docref){

  // Remember active layer and active channels.
  // We use try-catch here because if the currently selected 'channel'
  // is ONLY a mask, then docref.activeChannels is meaningless, since
  // a mask is not a channel. activeChannels gives a photoshop error in that
  // case.
  var AL = docref.activeLayer, AC;
  var ALvis = AL.visible;
  try {AC = docref.activeChannels}
  catch(e){AC = docref.componentChannels};

  var Vch = new Array();
  for (i=0;i<docref.channels.length;i++){
     Vch[i] = docref.channels[i].visible;
    }
};

// Remember visibility of each layer. A layerset is also considered as a
// layer, in that you can access it by using containeref.layers[i].
// We also set the visibility to all layers = true, because otherwise
// the shift may not always work. In some cases, the offset filter forgets
// to shift an invisible layer. Hence we make sure all layers are visible
// before shifting.
var V = new Array();
for (i=0;i<containeref.layers.length;i++){
  V[i] = containeref.layers[i].visible;
  
  // make all layers visible for shift
  // CS3: We only do this for invisible layers, in case layer does not permit
  // us to alter the visible property.
  if (!containeref.layers[i].visible) containeref.layers[i].visible = true;
};



// Select the bottommost layer as an initialization. Then when we select
// a different layer that is an adjustment layer, its layer mask
// will be automatically selected, so that the offset filter can then be
// applied to it.
docref.activeLayer = containeref.layers[containeref.layers.length-1];

for (i=0;i<containeref.layers.length;i++){
// If we have an undefined layer type, then it is probably a layer set.
// We skip that for now, because we will create a loop that explicitly 
// handles layer sets.
  if (typeof containeref.layers[i].kind == "undefined"){continue};

  docref.activeLayer = containeref.layers[i]; // Select layer i


// Select all channels, but only for a NORMAL layer.
// Actually, you only have to do this channel selection the FIRST time.
// When you then select another layer, then ALL the channels in that
// new layer are automatically selected. When all the component channels
// (R,G,B) are selected and then shifted by the offset filter, any
// layer mask present is automatically shifted along with the component
// channels.
//
// We ONLY select the component channels if the layer is a NORMAL layer.
// If we have an adjustment layer, then we do NOT want to select the component
// channels. It is important that we do not do that.
  if (docref.activeLayer.kind == LayerKind.NORMAL) {
      docref.activeChannels = docref.componentChannels;
    };

// For an adjustment layer, I wanted to explicitly select the mask, because
// this is necessary for the offset filter to operate on an adjustment layer.
// However, it is not clear to me how to explicitly select the mask on an 
// adjustment layer using javascript. Fortunately, the
// mask is automatically selected when the adjustment layer is selected 
// (assuming that a different layer is first selected, and THEN the adjustment
// layer is selected).

  // Deselect all, so we can shift the entire layer
  docref.selection.deselect();

  // offs() uses a try-catch statement, because if there is an empty layer 
  // and you try to shift, we get an error.
  offs(hshift,0);
//docref.activeLayer.applyOffset(hshift, 0, OffsetUndefinedAreas.WRAPAROUND);


  // Reselect tiny area to avoid histogram building
  docref.selection.select(selregion,SelectionType.REPLACE);

}


// create small selection to avoid histogram building
docref.selection.select(selregion,SelectionType.REPLACE);

// Process each of the layer sets RECURSIVELY.
// We also shift any mask associated with a layerset.
for (i=0;i<containeref.layerSets.length;i++){
  docref.activeLayer = containeref.layerSets[i];

  // Deselect all, so we can shift the entire layer
  docref.selection.deselect();

  // Make sure Mask (if it exists) is selected.
  // Then shift that mask.
  //
  // If Mask does not exist, THEN DO NOT call offset routine, or else
  // it will shift layers inside the layerset.
  // In other words, we get some unexpected behavior - 
  //   if mask exists for layerset, then offs() will shift it.
  //   if mask does not exist for layerset, then offs() will shift layers
  //     inside the layerset.
  // This is not desired, because we explicitly shift the layers inside
  // a layerset one by one anyway.
  if (selectmask()){  // Offset current layerset mask, if it exists
         offs(hshift,0)};

  // Shift all layers WITHIN layerset[i] by half their width
  shiftbyhalf(containeref.layerSets[i],hshift);
}


// When containeref refers to the entire document (NOT to a layerSet),
// then shift the ALPHA channels. Also restore the active layer, active
// channels, and the visibilities of the channels and layers.
if (containeref == docref){
   shiftalpha(selregion,hshift);

   docref.selection.select(selregion,SelectionType.REPLACE);

   // Restore active layer and active channels. Notice that the visibility
   // of the active layer is restored explicitly in addition to the loop
   // below that restores visibility for all top level layers. The reason
   // for this is in case AL points to a layer INSIDE a layerset, then
   // the loop below will not restore that visibility for AL.
   docref.activeLayer = AL;

   // Updated for CS3 to avoid error
   if (docref.activeLayer.visible != ALvis) docref.activeLayer.visible = ALvis;

   docref.activeChannels = AC;

   // Restore visibility of each layer in the top level of docref
   for (i=0;i<docref.layers.length;i++){

     // Updated for CS3 to avoid error
     if (docref.layers[i].visible != V[i]) docref.layers[i].visible = V[i];

   };

   // Restore visibility of each channel
   for (i=0;i<docref.channels.length;i++){
      docref.channels[i].visible = Vch[i]};


}
else{
   docref.selection.select(selregion,SelectionType.REPLACE);
   
   // Restore visibility of each layer within the layerset
   for (i=0;i<containeref.layers.length;i++){
     
     if (containeref.layers[i].visible != V[i]) containeref.layers[i].visible = V[i]

     };
    };


} // end of function


function selectrgb(){
var id31 = charIDToTypeID( "slct" );
var desc9 = new ActionDescriptor();
var id32 = charIDToTypeID( "null" );
var ref6 = new ActionReference();
var id33 = charIDToTypeID( "Chnl" );
var id34 = charIDToTypeID( "Chnl" );
var id35 = charIDToTypeID( "RGB " );
ref6.putEnumerated( id33, id34, id35 );
desc9.putReference( id32, ref6 );
executeAction( id31, desc9, DialogModes.NO );
}

// Select layer mask, if it exists
function selectmask(){
var id36 = charIDToTypeID( "slct" );
var desc10 = new ActionDescriptor();
var id37 = charIDToTypeID( "null" );
var ref7 = new ActionReference();
var id38 = charIDToTypeID( "Chnl" );
var id39 = charIDToTypeID( "Chnl" );
var id40 = charIDToTypeID( "Msk " );
ref7.putEnumerated( id38, id39, id40 );
desc10.putReference( id37, ref7 );
try{
  executeAction( id36, desc10, DialogModes.NO );
  return true
  }
catch(e){return false};

}


// Shift the alpha channels by half their width
function shiftalpha(selregion,hshift){
var docref = app.activeDocument;
docref.selection.select(selregion,SelectionType.REPLACE);

// If there are some non component channels, shift them as well?
// These are ALPHA channels we are referring to.
// If a channel is NOT a component color channel, then it is
// an ALPHA channel so then attempt to shift it.
if (docref.componentChannels.length < docref.channels.length){
 for (i=0;i<docref.channels.length;i++){
   if (docref.channels[i].kind != ChannelType.COMPONENT){
     docref.activeChannels = [docref.channels[i]];
     docref.selection.deselect();
     offs(hshift,0);
     docref.selection.select(selregion,SelectionType.REPLACE);
     }
   }
  }
}

function main(){
if (app.documents.length==0){
  alert("No documents open, script aborted");
  return;
   }

savestate(); // Save Photoshop State

// app.purge(PurgeTarget.ALLCACHES); // purge to speed up the script

docref = app.activeDocument;

// Check if document has odd or even width.
// If width is odd, then hshift will NOT be exactly half of width.
//
// We assume here that width/2 has no rounding errors when width is even.
// Photoshop Javascript appears to be smart enough to avoid that error.
// However, sometimes docref.width.value is not an integer (it is something
// like integer + 1e-13). I would consider this a Photoshop CS/CS2 bug.
// Hence, we must take Math.round(docref.width.value)
var width = Math.round(docref.width.value); // round to avoid rounding error
var hshift = Math.round(width/2);
if (hshift!=(width/2)){
  alert("Image width is an odd number of pixels, so\n" +
  "image shift cannot be EXACTLY width/2. Shift\n" +
  "will be (width/2) + 1/2 pixels in this case.\n")
 };


// Shift all layers by half their width
shiftbyhalf(docref,hshift);


// We deselect AFTER all the layer visibilities are restored. Otherwise
// Photoshop may try to build a histogram each time a layer visibility
// changes.
docref.selection.deselect();

restorestate(); // Restore Photoshop State
}

// -----------------------------------------------------

var G = new Object(); // Global object

main();




